@@ -1,5 +1,5 @@
-from datetime import datetime
-from distutils.version import LooseVersion
+from __future__ import annotations
+
 from itertools import product
 
 import numpy as np
@@ -23,9 +23,8 @@ from xarray.core.combine import (
     _infer_concat_order_from_positions,
     _new_tile_id,
 )
-
-from . import assert_equal, assert_identical, requires_cftime
-from .test_dataset import create_test_data
+from xarray.tests import assert_equal, assert_identical, requires_cftime
+from xarray.tests.test_dataset import create_test_data
 
 
 def assert_combined_tile_ids_equal(dict1, dict2):
@@ -229,8 +228,12 @@ class TestTileIDsFromCoords:
         assert concat_dims == ["simulation"]
 
     def test_datetime_coords(self):
-        ds0 = Dataset({"time": [datetime(2000, 3, 6), datetime(2001, 3, 7)]})
-        ds1 = Dataset({"time": [datetime(1999, 1, 1), datetime(1999, 2, 4)]})
+        ds0 = Dataset(
+            {"time": np.array(["2000-03-06", "2000-03-07"], dtype="datetime64[ns]")}
+        )
+        ds1 = Dataset(
+            {"time": np.array(["1999-01-01", "1999-02-04"], dtype="datetime64[ns]")}
+        )
 
         expected = {(0,): ds1, (1,): ds0}
         actual, concat_dims = _infer_concat_order_from_coords([ds0, ds1])
@@ -392,7 +395,7 @@ class TestNestedCombine:
 
     def test_combine_nested_join_exact(self):
         objs = [Dataset({"x": [0], "y": [0]}), Dataset({"x": [1], "y": [1]})]
-        with pytest.raises(ValueError, match=r"indexes along dimension"):
+        with pytest.raises(ValueError, match=r"cannot align.*join.*exact"):
             combine_nested(objs, concat_dim="x", join="exact")
 
     def test_empty_input(self):
@@ -747,7 +750,7 @@ class TestCombineDatasetsbyCoords:
 
     def test_combine_coords_join_exact(self):
         objs = [Dataset({"x": [0], "y": [0]}), Dataset({"x": [1], "y": [1]})]
-        with pytest.raises(ValueError, match=r"indexes along dimension"):
+        with pytest.raises(ValueError, match=r"cannot align.*join.*exact.*"):
             combine_nested(objs, concat_dim="x", join="exact")
 
     @pytest.mark.parametrize(
@@ -1141,22 +1144,17 @@ def test_combine_by_coords_raises_for_differing_calendars():
     da_1 = DataArray([0], dims=["time"], coords=[time_1], name="a").to_dataset()
     da_2 = DataArray([1], dims=["time"], coords=[time_2], name="a").to_dataset()
 
-    if LooseVersion(cftime.__version__) >= LooseVersion("1.5"):
-        error_msg = (
-            "Cannot combine along dimension 'time' with mixed types."
-            " Found:.*"
-            " If importing data directly from a file then setting"
-            " `use_cftime=True` may fix this issue."
-        )
-    else:
-        error_msg = r"cannot compare .* \(different calendars\)"
-
+    error_msg = (
+        "Cannot combine along dimension 'time' with mixed types."
+        " Found:.*"
+        " If importing data directly from a file then setting"
+        " `use_cftime=True` may fix this issue."
+    )
     with pytest.raises(TypeError, match=error_msg):
         combine_by_coords([da_1, da_2])
 
 
 def test_combine_by_coords_raises_for_differing_types():
-
     # str and byte cannot be compared
     da_1 = DataArray([0], dims=["time"], coords=[["a"]], name="a").to_dataset()
     da_2 = DataArray([1], dims=["time"], coords=[[b"b"]], name="a").to_dataset()
